#ifndef _ENTSCHEIDUNG_H_
#define _ENTSCHEIDUNG_H_

#if __GNUG__ >= 2
#  pragma interface
#endif

#include "weltmod.h"
#include "advancedskills.h"
#include "sensors.h"
#include "sensorik.h"
#include "sender.h"
#include "alinkli.h"
#include "positions.h"

//#define JAN_TEST


// States
#define STATE_UNKNOWN              0
#define STATE_INIT                 1
#define STATE_SEEK_BALL            2
#define STATE_FETCH_BALL           3
#define STATE_TAKE_INFLUENCE_POS   4
#define STATE_TAKE_REGULAR_POS     5
#define STATE_TAKE_WATCH_DIRECTION 6
#define STATE_SCAN_KICK_DIRECTION  7
#define STATE_KICK_BALL            8
#define STATE_COLLECT_INFOS        9
#define STATE_TURN_TO_BALL        10
#define STATE_DRIBBLE_BALL        11
#define STATE_PLACE_BESIDE_BALL   12
#define STATE_PLACE_NEAR_BALL   13
#define STATE_LOOK_FOR_BALL       14
#define STATE_FETCH_NEAR_BALL     15

// Konstanten fuer ComputeBestActiveBehaviour
#define DO_KICK                    1
#define DO_TURN                    2
#define DO_DRIBBLE                 3
#define DO_GOALKICK                4

// Konstanten fuer ComputeAllowedKickDegrees
#define MAX_DEGREE_DERIVATION   20.0 // ?

// Konstanten fuer ComputeBestKickDirection, ComputeBestWatchDirection, ComputeNextWatchDirection
#define WORST_DISTANCE          20.0 // ????
#define NUMBER_OF_OUR_PLAYERS   11.0
#define NUMBER_OF_THEIR_PLAYERS 11.0
#define BALL_KICK_RANGE         50.0 // ????
#define JAMM_RANGE               5.00
#define GOAL_KICK_POWER        400.0

#define DIRECTION_FRONT          2 // 90 Grad-Drehungen
#define NO_DIRECTION_LEFT        0
#define LEFT_DIRECTION           1
#define RIGHT_DIRECTION          2

// Konstanten fuer Think

// Jan
//#define THINK_RANGE           50.0
#define THINK_RANGE             35.0
#define INFLUENCE_PLAYER_STEPS  7
#define INFLUENCE_RANGE         20.0 
#define NEAR_RANGE              2.0
#define FREE_STEPS              2
#define GOALIE_CLEAR_RANGE      5.0

#define PLAYER_SELF             10
#define PLAYER_UNKNOWN          11

#define MAX_STEPS              500

#define NULL_POS               (NULL_VEKTOR)

// Jan

class Brain
{
public:
    Brain (Sensorik* sensorik, Sensors* sense, Sender* sender, Weltmodell* weltmod, AdvancedSkills* advskills);
    ~Brain ();
    
    void Think ();

private:
    void InitRole ();

    int ComputeBestActiveBehaviour (double bestkickvalue);

    double ComputeBestKickDirection (Koerper* bestdegree, Koerper* bestpower);
    // Rueckgabewert: Bewertung des besten Winkels, in bestdegree bester Winkel (relativ) in Grad

    int ComputeNextWatchDirection (Koerper* nextdegree);
    // Rueckgabewert: 1 falls erfolgreich, 0 falls keine weitere Richtung moeglich ist
    // Rueckgabewert in nextdegree: naechster Sichtwinkel (relativ) in Grad

    int WhichDirection ();
    // Rueckgabewert: RIGHT_DIRECTION, LEFT_DIRECTION, NO_DIRECTION_LEFT
    
    Punkt ComputePosBesideBall (Punkt pos_ball, double distance, Punkt pos_self);
    // berechnet die Position neben dem Ball mit Abstand distance im Sinne der Vorzugsrichtung

    int ComputeAllowedKickDegrees (Koerper* leftdegree, Koerper* rightdegree);
    // Rueckgabewert in leftdegree: linker erlaubter Winkel zum Kicken, in rightdegree: rechter ...
    // Rueckgabewert: 1 es gibt erlaubte Winkel, 0 es gibt keine erlaubten Winkel
    // alle Winkel relativ in Grad

// Jan
    int isNear(Punkt point, Punkt otherPoint);
    int decideNew();
    void computeForAllStepsToBall();
    int computeStepsToBall(int team, int player, int freeSteps = 0);
    int neighbours(Spieler playerWithBall, Spieler playerInfluence);
    Punkt computeBestPosToWaitForBall(int playerNumber);
    double computeQualityOfPos(Punkt point, int playerNumber);
    double computeFreedom(Punkt point, Spieler player);
// Jan

    int m_nNextLeftDirection, m_nNextRightDirection;
    Vektor m_cBestWatchDirection;
    int m_nRegion;
    int m_nKickCounter;
    
    // Rollenparameter
    double m_fWatchDerivation;
    double m_fDribbleFactor;
    double m_fAcceptance;
    double m_fMeanFactor;
    double m_fBestFactor;
    double m_fKickPower;
    
    int m_nState;

// Jan
    Punkt interceptPos;
    int lookNewTime;
    int wantedPosReached;
    int stepsOurPlayer[12], stepsTheirPlayer[12]; // Schritte die der Spieler zum Ball braucht;
    int minStepsOur, minStepsTheir;               // Spieler mit jeweils minimalen Schritten zum Ball;
    int minSteps;                      // Schritte in denen der schnellste Spieler den Ball erreichen kann;

// Jan

    Sensorik* m_cSensorik;
    Sensors* m_cSense;
    Sender* m_cSender;
    Weltmodell* m_cWeltmodell;
    AdvancedSkills* m_cAdvSkills;
};

#endif
