#include <stdio.h>
#include <string.h>
#include <stream.h>

#include "sender.h"
#include "weltmod.h"
#include "Configuration.h"

extern Weltmodell weltmodell;

Sender::Sender(BaseSkills *skills, int queue_size=10){
  myskills=skills;
  queue_len=queue_size;
  cmd_queue=new cmd_struct[queue_len];
  index_w=0;
  index_r=0;
  InitializeQueue();
}

void Sender::InitializeQueue(){
  cmd_queue[index_r].cmd=NOTHING; // Falls zwischendurch ein Kommando abgesetzt wird
  for(int i=0; i<queue_len; i++){
    cmd_queue[i].cmd=NOTHING;
    cmd_queue[i].status=IGNORE;
  }
  index_w=0;
  index_r=0;
}

void Sender::SetCommand(int cmd, float arg, int stat=IGNORE){
  cmd_queue[index_w].cmd=cmd;
  cmd_queue[index_w].farg1=arg;
  cmd_queue[index_w].status=stat;
  index_w=((index_w+1) % queue_len);
}

void Sender::SetCommand(int cmd, float arg1, float arg2, int stat=IGNORE){
  cmd_queue[index_w].cmd=cmd;
  cmd_queue[index_w].farg1=arg1;
  cmd_queue[index_w].farg2=arg2;
  cmd_queue[index_w].status=stat;
  index_w=((index_w+1) % queue_len);
}

void Sender::SetCommand(int cmd, int arg1, int arg2, int stat=IGNORE){
  cmd_queue[index_w].cmd=cmd;
  cmd_queue[index_w].iarg1=arg1;
  cmd_queue[index_w].iarg2=arg2;
  cmd_queue[index_w].status=stat;
  index_w=((index_w+1) % queue_len);  
}

void Sender::SetCommand(int cmd, char *arg, int stat=IGNORE){
  cmd_queue[index_w].cmd=cmd;
  cmd_queue[index_w].text=(char *) new char[strlen(arg)];
  strcpy(cmd_queue[index_w].text,arg);
  cmd_queue[index_w].status=stat;
  index_w=((index_w+1) % queue_len);  
}

void Sender::SendCommand(){

  if(index_w != index_r){
    switch(cmd_queue[index_r].cmd){
    case MOVETO :
      myskills->MoveTo(cmd_queue[index_r].farg1, cmd_queue[index_r].farg2);
      break;
    case TURN :
      myskills->Turn(cmd_queue[index_r].farg1);
      break;
    case DASH :
      if (weltmodell.frage_selbst().frage_stamina() - cmd_queue[index_r].farg1 <= globalConf->staminaDashIgnoreThr){ 
	cmd_queue[index_r].farg1=weltmodell.frage_selbst().frage_stamina() - globalConf->staminaDashIgnoreThr;
      }
      myskills->Dash(cmd_queue[index_r].farg1);
      break;
    case KICK :
      myskills->Kick(cmd_queue[index_r].farg1, cmd_queue[index_r].farg2);
      break;
    case CHANGEVIEW :
      myskills->ChangeView(cmd_queue[index_r].iarg1,cmd_queue[index_r].iarg2);
      break;
    case SAY :
      myskills->Say(cmd_queue[index_r].text);
      delete cmd_queue[index_r].text;
      break;
    }
    if(cmd_queue[index_r].cmd!=NOTHING){ // habe etwas getan

      update_by_command(cmd_queue[index_r]);

      cmd_queue[index_r].cmd=NOTHING; // zuruecksetzen
      index_r=((index_r+1) % queue_len);
    }
  }
  else{ // nichts zu tun
	     
    // Falls beide Indize gleich -- vieleicht wurde schon begonnen 
    // etwas einzutragen; der naechste Eintrag in cmd_queue ist aber
    // auf jeden Fall auf NOTHING gesetzt - darum der merkwuerdige Index

    //update_by_command (cmd_queue[(index_r+1) % queue_len]);

  }
}







