#ifndef _SPIELFELD

#define _SPIELFELD

#include"fussbal.h"
#include"excepti.h"
#include"sensors.h"		// entspricht sensor.h

// Das Koordinatensystem verlauft in positiver x-Richtung nach rechts
// und in positiver y-Richtung nach oben.

typedef Koerper Koord;
const Koord Koord_undef = Koerper_Max; // undefinierter Wert

typedef Vektor Punkt;
	// x1 entspricht x-, x2 y-Richtung

struct Koordinate
	{
	Punkt		p;
	Winkel		richtung;	// Winkel im Bogenmas

	void ausgeben(void);	// Ausgabe auf Bildschirm
	};

struct Relative_Koordinate
	{
	Koord		d;			// Abstand
	Winkel		richtung;		// relativer Winkel

	void ausgeben(void);	// Ausgabe auf Bildschirm
	};

// Linien
const int anzahl_linien = 5;
const int obere_feldlinie = 0;
const int untere_feldlinie = 1;
const int linke_feldlinie = 2;
const int rechte_feldlinie = 3;
const int mittlere_feldlinie = 4;
const int undef_feldlinie = -1; // die Identifikation der Feldlinie ist nicht moglich.

const int horizontale_feldlinie = 1;
const int vertikale_feldlinie = 2; // parallel zur x- bzw. y-Achse verlaufende Linien

struct Feldlinie	// eine der das Fusballfeld begrenzenden Linien
	{
	int			nummer;	// Konstanten s. oben
	Punkt		p1,p2;	// der Anfangs- bzw. Endpunkt der Linie.

	inline int ist_horizontal(void) const // liefert != 0, falls Linie in x-Richtung verlauft
		{return p1.x2 == p2.x2;}
	inline int ist_vertikal(void) const // liefert != 0, falls Linie in y-Richtung verlauft
		{return p1.x1 == p2.x1;}
	};

// Flaggen
const int anzahl_flaggen = 12;
const int flagge_links_oben = 0;
const int flagge_mitte_oben = 1;
const int flagge_rechts_oben = 2;
const int flagge_links_unten = 3;
const int flagge_mitte_unten = 4;
const int flagge_rechts_unten = 5;
const int flagge_s_links_oben = 6;	// Strafraumflaggen
const int flagge_s_links_mitte = 7;
const int flagge_s_links_unten = 8;
const int flagge_s_rechts_oben = 9;
const int flagge_s_rechts_mitte = 10;
const int flagge_s_rechts_unten = 11;
const int flagge_undef = -1;	// unidentifizierte Flagge

struct Flagge
	{
	int			nummer; // Konstenten s. oben
	Punkt		p;	// Position der Flagge
	};

// Tore
const int anzahl_tore = 2;
const int rechtes_tor = 0;
const int linkes_tor = 1;
const int undef_tor = -1; // unidentifiziertes Tor

struct Tor
	{
	int			nummer; // Konstanten s. oben
	Punkt		p; // Position des Tors
	};

// Parteien
const char linke_seite = 'l';	// unser Tor ist links
const char rechte_seite = 'r'; // unser Tor ist rechts
const char undef_seite = '?';	// unidentifizierte Seite

class Spielfeld
	{
private:
	static const Koerper feldbreite,feldlaenge,strafraumbreite,strafraumlaenge;
		// Mase des Spielfeldes
	static const Flagge flaggen[anzahl_flaggen];
	static const Feldlinie linien[anzahl_linien];		
	static const Tor tore[anzahl_tore];

public:
	inline Koord frage_laenge(void) const // liefert die Laenge des Spiefeldes
		{return feldlaenge;}
	inline Koord frage_breite(void) const
		{return feldbreite;}
		// Laenge ist die Ausdehnung in x-Richtung, Breite die in y-Richtung.
	inline int ist_im_spielfeld(Punkt p) const // liefert != 0, falls Punkt im Spielfeld liegt, sonst 0
		{return betrag(p.x1) <= feldlaenge / 2 && betrag(p.x2) <= feldbreite / 2;}
	
	inline const Flagge &frage_flagge(int nummer) const // liefert die Flagge mit der angegebenen Nummer
		{
		if(nummer < 0 || nummer >= anzahl_flaggen) exception("Spielfeld::frage_flagge: Unzulassiger Parameter");
		return flaggen[nummer];
		}

	inline const Feldlinie &frage_linie(int nummer) const // liefert die Linie mit der angegebenen Nummer
		{
		if(nummer < 0 || nummer >= anzahl_linien) exception("Spielfeld::frage_linie: Unzulassiger Parameter");
		return linien[nummer];
		}
	
	inline const Tor &frage_tor(int nummer) const // liefert das angegebene Tor
		{
		if(nummer != linkes_tor && nummer != rechtes_tor) exception("Spielfeld::frage_tor: Unzulassiger Parameter");
		return tore[nummer];
		}
	
        double computeFaceDir( const Feldlinie &linie1,const Feldlinie &linie2,
                 const Relative_Koordinate &delta_linie1,
		 const Relative_Koordinate &delta_linie2,int count ) const;
	         // Berechnet die absolute Blickrichtung. Es wird eine der 
		 // uebergebenen Linien ausgewaehlt, um daraus den Blickwinkel 
		 // zu bestimmen. 

int triangulateViewPoint( const Feldlinie &linie1,const Feldlinie &linie2,
	const Relative_Koordinate &delta_linie1,const Relative_Koordinate &delta_linie2,
	int linien_count,
	const int id[10],const Relative_Koordinate delta_p[10],
	int punkt_count, Koordinate &S) const;
	
Vektor computePosFromFlag( const int &id,const Relative_Koordinate &delta_p,
				  double faceDir ) const;
				  
	int berechne_koordinate(const Feldlinie &linie,const Relative_Koordinate &delta_linie,
		const Punkt &p,const Relative_Koordinate &delta_p,Koordinate &S) const;
		// Berechnet aus den gegebenen relativen Angaben die absolute Position eines Objektes.
		// Im Erfolgsfall wird != 0, sonst 0 geliefert.
		// Der Spieler steht innerhalb des Feldes.
		
	int berechne_koordinate(const Feldlinie &linie1,const Feldlinie &linie2,
		const Relative_Koordinate &delta_linie1,const Relative_Koordinate &delta_linie2,
		const Punkt &p,const Relative_Koordinate &delta_p,Koordinate &S) const;
		// Berechnet aus den gegebenen relativen Angaben die absolute Position eines Objektes.
		// Im Erfolgsfall wird != 0, sonst 0 geliefert.
		// Der Spieler steht auserhalb des Feldes.

	int berechne_relative_koordinate(const Feldlinie &linie,const Koordinate &S,Relative_Koordinate &delta) const;
		// berechnet die relative Koordinate eines Punktes bzgl. einer Linie
		// Im Erfolgsfall wird != 0 geliefert, sonst 0
	int berechne_relative_koordinate(const Punkt &p,const Koordinate &S,Relative_Koordinate &delta) const;
		// berechnet die relative Koordinate eines Punktes bzgl. eines Punktes
		// Im Erfolgsfall wird != 0 geliefert, sonst 0
	
	int transformiere(RelPos &rp,Relative_Koordinate &rk);
		// RelPos in Relative_Koordinate. Liefert 0, falls eine der Angaben nicht gesetzt war. ('valid')
	void transformiere(Relative_Koordinate &rk,RelPos &rp);
		// umgekehrt
	};

#endif
